@@ -7,9 +7,9 @@
 #include "vm/types.h"
 
 enum machine_reg {
-	R0,
-	R1,
-	R2,
+	MACH_REG_R0,
+	MACH_REG_R1,
+	MACH_REG_R2,
 
 	/* Number of general purpose registers.  */
 	NR_GP_REGISTERS,
@@ -20,7 +20,10 @@ enum machine_reg {
 	/* The above registers are available for get_fixed_var().  */
 	NR_FIXED_REGISTERS = NR_REGISTERS,
 
-	MACH_REG_UNASSIGNED = INT_MAX,
+	MACH_REG_UNASSIGNED = CHAR_MAX,
+};
+
+struct register_state {
 };
 
 #define NR_CALLER_SAVE_REGS 0
@@ -33,4 +36,9 @@ static inline bool reg_supports_type(enum machine_reg reg, enum vm_type type)
 
 #define GPR_VM_TYPE J_LONG
 
+static inline enum vm_type reg_default_type(enum machine_reg reg)
+{
+	return GPR_VM_TYPE;
+}
+
 #endif /* __JIT_REGISTERS_H */
