@@ -35,11 +35,14 @@
 
 #include "lib/buffer.h"
 
+#include "vm/types.h"
 #include "vm/die.h"
 
 #include <inttypes.h>
 #include <stdint.h>
 
+#define INVALID_INSN		0
+
 struct x86_insn {
 	uint64_t		flags;
 	uint8_t			rex_prefix;
@@ -152,7 +155,7 @@ enum x86_insn_flags {
 	OPC_EXT			= (1ULL << 32),	/* The reg field of ModR/M byte provides opcode extension */
 	OPC_REG			= (1ULL << 33), /* The opcode byte also provides operand register */
 
-	NO_REX_W		= (1ULL << 34), /* No REX W prefix needed */
+	REX_W_PREFIX		= (1ULL << 34), /* REX W prefix supported */
 	INDEX			= (1ULL << 35),
 	OPERAND_SIZE_PREFIX	= (1ULL << 36), /* Operand-size override prefix */
 
@@ -199,28 +202,29 @@ enum x86_addmode {
 #define OPCODE_EXT_MASK		(0xe000000000000000ULL)
 
 static uint64_t encode_table[NR_INSN_TYPES] = {
-	[INSN_ADC_IMM_REG]		= OPCODE(0x81) | OPCODE_EXT(2)   | ADDMODE_IMM_REG | WIDTH_FULL,
-	[INSN_ADC_MEMBASE_REG]		= OPCODE(0x13) | ADDMODE_RM_REG  | WIDTH_FULL,
-	[INSN_ADC_REG_REG]		= OPCODE(0x11) | ADDMODE_REG_REG | DIR_REVERSED | WIDTH_FULL,
+	[INSN_ADC_IMM_REG]		= OPCODE(0x81) | OPCODE_EXT(2)   | ADDMODE_IMM_REG | WIDTH_FULL | REX_W_PREFIX,
+	[INSN_ADC_MEMBASE_REG]		= OPCODE(0x13) | ADDMODE_RM_REG  | WIDTH_FULL | REX_W_PREFIX,
+	[INSN_ADC_REG_REG]		= OPCODE(0x11) | ADDMODE_REG_REG | DIR_REVERSED | WIDTH_FULL | REX_W_PREFIX,
 	[INSN_ADDSD_MEMDISP_XMM]	= REPNE_PREFIX | ESCAPE_OPC_BYTE | OPCODE(0x58) | ADDMODE_MEMDISP_REG | WIDTH_64,
 	[INSN_ADDSD_XMM_XMM]		= REPNE_PREFIX | ESCAPE_OPC_BYTE | OPCODE(0x58) | ADDMODE_REG_REG | WIDTH_64,
 	[INSN_ADDSS_XMM_XMM]		= REPE_PREFIX  | ESCAPE_OPC_BYTE | OPCODE(0x58) | ADDMODE_REG_REG | WIDTH_FULL,
-	[INSN_ADD_IMM_REG]		= OPCODE(0x81) | OPCODE_EXT(0)   | ADDMODE_IMM_REG | WIDTH_FULL,
-	[INSN_ADD_MEMBASE_REG]		= OPCODE(0x03) | ADDMODE_RM_REG  | WIDTH_FULL,
-	[INSN_ADD_REG_REG]		= OPCODE(0x01) | ADDMODE_REG_REG | DIR_REVERSED | WIDTH_FULL,
-	[INSN_AND_MEMBASE_REG]		= OPCODE(0x23) | ADDMODE_RM_REG  | WIDTH_FULL,
-	[INSN_AND_REG_REG]		= OPCODE(0x21) | ADDMODE_REG_REG | DIR_REVERSED | WIDTH_FULL,
+	[INSN_ADD_IMM_REG]		= OPCODE(0x81) | OPCODE_EXT(0)   | ADDMODE_IMM_REG | WIDTH_FULL | REX_W_PREFIX,
+	[INSN_ADD_MEMBASE_REG]		= OPCODE(0x03) | ADDMODE_RM_REG  | WIDTH_FULL | REX_W_PREFIX,
+	[INSN_ADD_REG_REG]		= OPCODE(0x01) | ADDMODE_REG_REG | DIR_REVERSED | WIDTH_FULL | REX_W_PREFIX,
+	[INSN_AND_MEMBASE_REG]		= OPCODE(0x23) | ADDMODE_RM_REG  | WIDTH_FULL | REX_W_PREFIX,
+	[INSN_AND_REG_REG]		= OPCODE(0x21) | ADDMODE_REG_REG | DIR_REVERSED | WIDTH_FULL | REX_W_PREFIX,
 	[INSN_CALL_REG]			= OPCODE(0xFF) | OPCODE_EXT(2)   | ADDMODE_RM | WIDTH_FULL,
-	[INSN_CLTD_REG_REG]		= OPCODE(0x99) | WIDTH_FULL,
-	[INSN_CMP_IMM_REG]		= OPCODE(0x81) | OPCODE_EXT(7)   | ADDMODE_IMM_REG | WIDTH_FULL,
-	[INSN_CMP_MEMBASE_REG]		= OPCODE(0x3b) | ADDMODE_RM_REG  | WIDTH_FULL,
-	[INSN_CMP_REG_REG]		= OPCODE(0x39) | ADDMODE_REG_REG | DIR_REVERSED | WIDTH_FULL,
+	[INSN_CLTD_REG_REG]		= OPCODE(0x99) | WIDTH_FULL | REX_W_PREFIX,
+	[INSN_CMP_IMM_REG]		= OPCODE(0x81) | OPCODE_EXT(7)   | ADDMODE_IMM_REG | WIDTH_FULL | REX_W_PREFIX,
+	[INSN_CMP_MEMBASE_REG]		= OPCODE(0x3b) | ADDMODE_RM_REG  | WIDTH_FULL | REX_W_PREFIX,
+	[INSN_CMP_REG_REG]		= OPCODE(0x39) | ADDMODE_REG_REG | DIR_REVERSED | WIDTH_FULL | REX_W_PREFIX,
 	[INSN_DIVSD_XMM_XMM]		= REPNE_PREFIX | ESCAPE_OPC_BYTE | OPCODE(0x5e) | ADDMODE_REG_REG | WIDTH_64,
 	[INSN_DIVSS_XMM_XMM]		= REPE_PREFIX  | ESCAPE_OPC_BYTE | OPCODE(0x5e) | ADDMODE_REG_REG | WIDTH_FULL,
 	[INSN_FLD_64_MEMLOCAL]		= OPCODE(0xdd) | OPCODE_EXT(0)   | ADDMODE_MEMLOCAL | WIDTH_64,
 	[INSN_FLD_MEMLOCAL]		= OPCODE(0xd9) | OPCODE_EXT(0)   | ADDMODE_MEMLOCAL | WIDTH_FULL,
 	[INSN_FSTP_64_MEMLOCAL]		= OPCODE(0xdd) | OPCODE_EXT(3)   | ADDMODE_MEMLOCAL | WIDTH_64,
 	[INSN_FSTP_MEMLOCAL]		= OPCODE(0xd9) | OPCODE_EXT(3)   | ADDMODE_MEMLOCAL | WIDTH_FULL,
+	[INSN_IC_CALL]			= INVALID_INSN,
 	[INSN_JMP_MEMBASE]		= OPCODE(0xff) | OPCODE_EXT(4)   | ADDMODE_RM | WIDTH_FULL,
 	[INSN_JMP_MEMINDEX]		= OPCODE(0xff) | OPCODE_EXT(4)   | ADDMODE_RM | INDEX | WIDTH_FULL,
 	[INSN_MOVSD_MEMBASE_XMM]	= REPNE_PREFIX | ESCAPE_OPC_BYTE | OPCODE(0x10) | ADDMODE_RM_REG | WIDTH_64,
@@ -237,42 +241,43 @@ static uint64_t encode_table[NR_INSN_TYPES] = {
 	[INSN_MOVSS_XMM_MEMDISP]	= REPE_PREFIX  | ESCAPE_OPC_BYTE | OPCODE(0x11) | ADDMODE_REG_MEMDISP | WIDTH_64,
 	[INSN_MOVSS_XMM_MEMLOCAL]	= REPE_PREFIX  | ESCAPE_OPC_BYTE | OPCODE(0x11) | ADDMODE_REG_MEMLOCAL | WIDTH_FULL,
 	[INSN_MOVSS_XMM_XMM]		= REPE_PREFIX  | ESCAPE_OPC_BYTE | OPCODE(0x10) | ADDMODE_REG_REG | WIDTH_FULL,
-	[INSN_MOVSX_16_MEMBASE_REG]	= OPCODE(0xbf) | ESCAPE_OPC_BYTE | ADDMODE_REG_REG | WIDTH_FULL,
-	[INSN_MOVSX_16_REG_REG]		= OPCODE(0xbf) | ESCAPE_OPC_BYTE | ADDMODE_REG_REG | WIDTH_FULL,
-	[INSN_MOVSX_8_MEMBASE_REG]	= OPCODE(0xbe) | ESCAPE_OPC_BYTE | ADDMODE_REG_REG | WIDTH_FULL,
-	[INSN_MOVSX_8_REG_REG]		= OPCODE(0xbe) | ESCAPE_OPC_BYTE | ADDMODE_REG_REG | WIDTH_FULL,
-	[INSN_MOVZX_16_REG_REG]		= OPCODE(0xb7) | ESCAPE_OPC_BYTE | ADDMODE_REG_REG | WIDTH_FULL,
-	[INSN_MOV_MEMBASE_REG]		= OPCODE(0x8b) | ADDMODE_RM_REG  | WIDTH_FULL,
-	[INSN_MOV_REG_MEMBASE]		= OPCODE(0x89) | ADDMODE_REG_RM  | WIDTH_FULL,
-	[INSN_MOV_REG_REG]		= OPCODE(0x89) | ADDMODE_REG_REG | DIR_REVERSED | WIDTH_FULL,
+	[INSN_MOVSX_16_MEMBASE_REG]	= OPCODE(0xbf) | ESCAPE_OPC_BYTE | ADDMODE_REG_REG | WIDTH_FULL | REX_W_PREFIX,
+	[INSN_MOVSX_16_REG_REG]		= OPCODE(0xbf) | ESCAPE_OPC_BYTE | ADDMODE_REG_REG | WIDTH_FULL | REX_W_PREFIX,
+	[INSN_MOVSX_8_MEMBASE_REG]	= OPCODE(0xbe) | ESCAPE_OPC_BYTE | ADDMODE_REG_REG | WIDTH_FULL | REX_W_PREFIX,
+	[INSN_MOVSX_8_REG_REG]		= OPCODE(0xbe) | ESCAPE_OPC_BYTE | ADDMODE_REG_REG | WIDTH_FULL | REX_W_PREFIX,
+	[INSN_MOVZX_16_REG_REG]		= OPCODE(0xb7) | ESCAPE_OPC_BYTE | ADDMODE_REG_REG | WIDTH_FULL | REX_W_PREFIX,
+	[INSN_MOV_MEMBASE_REG]		= OPCODE(0x8b) | ADDMODE_RM_REG  | WIDTH_FULL | REX_W_PREFIX,
+	[INSN_MOV_REG_MEMBASE]		= OPCODE(0x89) | ADDMODE_REG_RM  | WIDTH_FULL | REX_W_PREFIX,
+	[INSN_MOV_REG_REG]		= OPCODE(0x89) | ADDMODE_REG_REG | DIR_REVERSED | WIDTH_FULL | REX_W_PREFIX,
 	[INSN_MULSD_MEMDISP_XMM]	= REPNE_PREFIX | ESCAPE_OPC_BYTE | OPCODE(0x59) | ADDMODE_MEMDISP_REG | WIDTH_64,
 	[INSN_MULSD_XMM_XMM]		= REPNE_PREFIX | ESCAPE_OPC_BYTE | OPCODE(0x59) | ADDMODE_REG_REG | WIDTH_64,
 	[INSN_MULSS_XMM_XMM]		= REPE_PREFIX  | ESCAPE_OPC_BYTE | OPCODE(0x59) | ADDMODE_REG_REG | WIDTH_FULL,
-	[INSN_NEG_REG]			= OPCODE(0xf7) | OPCODE_EXT(3)   | ADDMODE_REG     | DIR_REVERSED | WIDTH_FULL,
-	[INSN_OR_MEMBASE_REG]		= OPCODE(0x0b) | ADDMODE_RM_REG  | WIDTH_FULL,
-	[INSN_OR_REG_REG]		= OPCODE(0x09) | ADDMODE_REG_REG | DIR_REVERSED | WIDTH_FULL,
-	[INSN_POP_MEMLOCAL]		= OPCODE(0x8f) | OPCODE_EXT(0)   | ADDMODE_MEMLOCAL| WIDTH_FULL | NO_REX_W,
-	[INSN_POP_REG]			= OPCODE(0x58) | OPC_REG         | ADDMODE_REG     | DIR_REVERSED | WIDTH_FULL | NO_REX_W,
-	[INSN_PUSH_MEMLOCAL]		= OPCODE(0xff) | OPCODE_EXT(6)   | ADDMODE_MEMLOCAL| WIDTH_FULL | NO_REX_W,
-	[INSN_PUSH_REG]			= OPCODE(0x50) | OPC_REG         | ADDMODE_REG     | WIDTH_FULL | NO_REX_W,
+	[INSN_NEG_REG]			= OPCODE(0xf7) | OPCODE_EXT(3)   | ADDMODE_REG     | DIR_REVERSED | WIDTH_FULL | REX_W_PREFIX,
+	[INSN_OR_MEMBASE_REG]		= OPCODE(0x0b) | ADDMODE_RM_REG  | WIDTH_FULL | REX_W_PREFIX,
+	[INSN_OR_REG_REG]		= OPCODE(0x09) | ADDMODE_REG_REG | DIR_REVERSED | WIDTH_FULL | REX_W_PREFIX,
+	[INSN_PHI]			= INVALID_INSN,
+	[INSN_POP_MEMLOCAL]		= OPCODE(0x8f) | OPCODE_EXT(0)   | ADDMODE_MEMLOCAL| WIDTH_FULL,
+	[INSN_POP_REG]			= OPCODE(0x58) | OPC_REG         | ADDMODE_REG     | DIR_REVERSED | WIDTH_FULL,
+	[INSN_PUSH_MEMLOCAL]		= OPCODE(0xff) | OPCODE_EXT(6)   | ADDMODE_MEMLOCAL| WIDTH_FULL,
+	[INSN_PUSH_REG]			= OPCODE(0x50) | OPC_REG         | ADDMODE_REG     | WIDTH_FULL,
 	[INSN_RET]			= OPCODE(0xc3) | ADDMODE_IMPLIED,
-	[INSN_SAR_IMM_REG]		= OPCODE(0xc1) | OPCODE_EXT(7)   | ADDMODE_IMM8_REG | DIR_REVERSED | WIDTH_FULL,
-	[INSN_SAR_REG_REG]		= OPCODE(0xd3) | OPCODE_EXT(7)   | ADDMODE_REG_REG | DIR_REVERSED | WIDTH_FULL,
-	[INSN_SBB_IMM_REG]		= OPCODE(0x81) | OPCODE_EXT(3)   | ADDMODE_IMM_REG | WIDTH_FULL,
-	[INSN_SBB_MEMBASE_REG]		= OPCODE(0x1b) | ADDMODE_RM_REG  | WIDTH_FULL,
-	[INSN_SBB_REG_REG]		= OPCODE(0x19) | ADDMODE_REG_REG | DIR_REVERSED | WIDTH_FULL,
-	[INSN_SHL_REG_REG]		= OPCODE(0xd3) | OPCODE_EXT(4)   | ADDMODE_REG_REG|DIR_REVERSED | WIDTH_FULL,
-	[INSN_SHR_REG_REG]		= OPCODE(0xd3) | OPCODE_EXT(5)   | ADDMODE_REG_REG|DIR_REVERSED | WIDTH_FULL,
+	[INSN_SAR_IMM_REG]		= OPCODE(0xc1) | OPCODE_EXT(7)   | ADDMODE_IMM8_REG | DIR_REVERSED | WIDTH_FULL | REX_W_PREFIX,
+	[INSN_SAR_REG_REG]		= OPCODE(0xd3) | OPCODE_EXT(7)   | ADDMODE_REG_REG | DIR_REVERSED | WIDTH_FULL | REX_W_PREFIX,
+	[INSN_SBB_IMM_REG]		= OPCODE(0x81) | OPCODE_EXT(3)   | ADDMODE_IMM_REG | WIDTH_FULL | REX_W_PREFIX,
+	[INSN_SBB_MEMBASE_REG]		= OPCODE(0x1b) | ADDMODE_RM_REG  | WIDTH_FULL | REX_W_PREFIX,
+	[INSN_SBB_REG_REG]		= OPCODE(0x19) | ADDMODE_REG_REG | DIR_REVERSED | WIDTH_FULL | REX_W_PREFIX,
+	[INSN_SHL_REG_REG]		= OPCODE(0xd3) | OPCODE_EXT(4)   | ADDMODE_REG_REG|DIR_REVERSED | WIDTH_FULL | REX_W_PREFIX,
+	[INSN_SHR_REG_REG]		= OPCODE(0xd3) | OPCODE_EXT(5)   | ADDMODE_REG_REG|DIR_REVERSED | WIDTH_FULL | REX_W_PREFIX,
 	[INSN_SUBSD_XMM_XMM]		= REPNE_PREFIX | ESCAPE_OPC_BYTE | OPCODE(0x5c) | ADDMODE_REG_REG | WIDTH_64,
 	[INSN_SUBSS_XMM_XMM]		= REPE_PREFIX  | ESCAPE_OPC_BYTE | OPCODE(0x5c) | ADDMODE_REG_REG | WIDTH_FULL,
-	[INSN_SUB_IMM_REG]		= OPCODE(0x81) | OPCODE_EXT(5)   | ADDMODE_IMM_REG | WIDTH_FULL,
-	[INSN_SUB_MEMBASE_REG]		= OPCODE(0x2b) | ADDMODE_RM_REG  | WIDTH_FULL,
-	[INSN_SUB_REG_REG]		= OPCODE(0x29) | ADDMODE_REG_REG | DIR_REVERSED | WIDTH_FULL,
-	[INSN_TEST_MEMBASE_REG]		= OPCODE(0x85) | ADDMODE_RM_REG  | WIDTH_FULL,
+	[INSN_SUB_IMM_REG]		= OPCODE(0x81) | OPCODE_EXT(5)   | ADDMODE_IMM_REG | WIDTH_FULL | REX_W_PREFIX,
+	[INSN_SUB_MEMBASE_REG]		= OPCODE(0x2b) | ADDMODE_RM_REG  | WIDTH_FULL | REX_W_PREFIX,
+	[INSN_SUB_REG_REG]		= OPCODE(0x29) | ADDMODE_REG_REG | DIR_REVERSED | WIDTH_FULL | REX_W_PREFIX,
+	[INSN_TEST_MEMBASE_REG]		= OPCODE(0x85) | ADDMODE_RM_REG  | WIDTH_FULL | REX_W_PREFIX,
 	[INSN_XORPD_XMM_XMM]		= OPERAND_SIZE_PREFIX | ESCAPE_OPC_BYTE | OPCODE(0x57) | ADDMODE_REG_REG | WIDTH_FULL,
 	[INSN_XORPS_XMM_XMM]		= ESCAPE_OPC_BYTE | OPCODE(0x57) | ADDMODE_REG_REG | WIDTH_FULL,
-	[INSN_XOR_MEMBASE_REG]		= OPCODE(0x33) | ADDMODE_RM_REG  | WIDTH_FULL,
-	[INSN_XOR_REG_REG]		= OPCODE(0x31) | ADDMODE_REG_REG | DIR_REVERSED | WIDTH_FULL,
+	[INSN_XOR_MEMBASE_REG]		= OPCODE(0x33) | ADDMODE_RM_REG  | WIDTH_FULL | REX_W_PREFIX,
+	[INSN_XOR_REG_REG]		= OPCODE(0x31) | ADDMODE_REG_REG | DIR_REVERSED | WIDTH_FULL | REX_W_PREFIX,
 };
 
 static inline bool is_imm_8(int32_t imm)
@@ -387,10 +392,16 @@ static bool insn_need_sib(struct insn *self, uint64_t flags)
 
 	if (flags & INDEX)
 		return true;
+
 #ifdef CONFIG_X86_64
+	/*
+	 * FIXME: This is used to detect register indirect calls that use the
+	 *        r12 register.
+	 */
 	if (flags & DST_NONE && insn_uses_reg(self, MACH_REG_R12)) /* DST_NONE? */
 		return true;
 #endif
+
 	if (flags & OPC_EXT)
 		return false;
 
@@ -399,12 +410,23 @@ static bool insn_need_sib(struct insn *self, uint64_t flags)
 			return false;
 
 		reg	= mach_reg(&self->dest.base_reg);
+
+#ifdef CONFIG_X86_64
+		if (flags & DST_MEM_DISP_BYTE && reg == MACH_REG_R12)
+			return true;
+#endif
 	} else {
 		if (flags & (SRC_MEMDISP|SRC_MEMLOCAL))
 			return false;
 
 		reg	= mach_reg(&self->src.base_reg);
+
+#ifdef CONFIG_X86_64
+		if (flags & SRC_MEM_DISP_BYTE && reg == MACH_REG_R12)
+			return true;
+#endif
 	}
+
 	return reg == MACH_REG_xSP;
 }
 
@@ -522,20 +544,24 @@ static inline bool operand_is_reg_high(struct operand *operand)
 	return reg_num & 0x8;
 }
 
+static inline int operand_is_int64(struct operand *reg)
+{
+	enum vm_type vm_type = reg->reg.interval->var_info->vm_type;
+
+	return vm_type_is_int64(vm_type);
+}
+
 static inline bool operand_is_reg_64(struct operand *operand)
 {
 	if (!operand_is_reg(operand))
 		return false;
 
-	return !operand_is_xmm_reg(operand);
+	return operand_is_int64(operand);
 }
 
 static uint8_t insn_rex_operand_64(struct insn *self, uint64_t flags)
 {
-	if (insn_is_call(self) || insn_is_branch(self))
-		return 0;
-
-	if (flags & NO_REX_W)
+	if (!(flags & REX_W_PREFIX))
 		return 0;
 
 	if (operand_is_reg_64(&self->src) || operand_is_reg_64(&self->dest))
@@ -689,8 +715,8 @@ void insn_encode(struct insn *self, struct buffer *buffer, struct basic_block *b
 
 	encode		= encode_table[self->type];
 
-	if (encode == 0)
-		die("unrecognized instruction type %d", self->type);
+	if (encode == INVALID_INSN)
+		die("phi instruction in code emission stage");
 
 	insn	= (struct x86_insn) {
 		.opcode		= insn_opcode(self, encode),
