@@ -17,4 +17,7 @@ struct vm_object *float_to_object(jfloat value);
 struct vm_object *double_to_object(jdouble value);
 struct vm_object *jvalue_to_object(union jvalue *value, enum vm_type vm_type);
 
+int object_to_jvalue(void *field_ptr, enum vm_type type, struct vm_object *value);
+struct vm_class *vm_type_to_class(struct vm_object *classloader, struct vm_type_info *type_info);
+
 #endif /* JATO__VM_BOXING_H */
