@@ -45,6 +45,8 @@
 #include <errno.h>
 #include <stdio.h>
 
+pthread_key_t current_exec_env_key;
+
 __thread struct vm_exec_env *current_exec_env;
 
 static struct vm_object *main_thread_group;
@@ -77,7 +79,7 @@ static struct vm_thread *vm_thread_alloc(void)
 	thread->posix_id = -1;
 	thread->interrupted = false;
 	thread->waiting_mon = NULL;
-	thread->thread_state = THREAD_STATE_CONSISTENT;
+	thread->thread_state = VM_THREAD_STATE_CONSISTENT;
 	pthread_cond_init(&thread->park_cond, NULL);
 	pthread_mutex_init(&thread->park_mutex, NULL);
 	thread->unpark_called = false;
@@ -244,12 +246,19 @@ int init_threading(void)
 
 static struct vm_exec_env *alloc_exec_env(void)
 {
-	struct vm_exec_env *ee = vm_alloc(sizeof(struct vm_exec_env));
+	struct vm_exec_env *ee;
+
+	ee = vm_alloc(sizeof(struct vm_exec_env));
 	if (!ee)
 		return NULL;
 
-	ee->thread = NULL;
+	ee->thread			= NULL;
+	ee->exception			= NULL;
+	ee->trace_classloader_level	= 0;
 	INIT_LIST_HEAD(&ee->free_monitor_recs);
+	ee->in_safepoint	= false;
+	ee->trace_buffer = NULL;
+
 	return ee;
 }
 
